#ifndef __BOARD_H__
#define __BOARD_H__




// Global data structure for board
const int BOARD_EDGE = 16;
const int BOARD_SIZE = BOARD_EDGE * BOARD_EDGE;
const int NUMBER_OF_ROBOTS = 4;

// declare constants to later use instead of numbers to select values from theBoard
const int NUMBER_OF_ELEMENTS = 5;   // Number of elements for each piece, enumerated below
const int PIECE = 0;    // to reference the character, such as in: theBoard[ i][ PIECE]
const int LEFT = 1;     // to reference the left wall character, such as in: theBoard[ i][ LEFT]
const int ABOVE = 2;    // to reference the above wall character, such as in: theBoard[ i][ ABOVE]
const int RIGHT = 3;    // to reference the right wall character, such as in: theBoard[ i][ RIGHT]
const int BELOW = 4;    // to reference the below wall character, such as in: theBoard[ i][ BELOW]


class Board
{

public:
    Board();
    void readFromFile();
    void setObjective();
    void display();
    void displayObjective();
    void clearDestinationPieces();
    void moveRobot(int moveCount);
    
    void moveRobotUp(int robot);
    void moveRobotRight(int robot);
    void moveRobotLeft(int robot);
    void moveRobotDown(int robot);
    
    int getNumberOfDestinationPieces() const;
    int getNumberOfRobots() const;
    int getGoalRobot() const;
    char getDestinationLetter() const;
    
    bool isOver() const;
    bool isValidDirection(char direction) const;
    bool isOccupied(int position) const;




private:
    // theBoard itself is represented by a one-dimensional array.
    // The other entries are used to keep track of walls.  Each row
    // in theBoard array will contain the character to be displayed,
    // followed by any "walls" to be displayed around the displayed
    // character.  For instance the first couple of entries are be:
    //
    //       PIECE   LEFT ABOVE RIGHT BELOW
    //      ---0---  --1-  --2-  --3-  --4-
    //    0    .       |     _
    //    1    .             _
    //   ...
    //   255   .                   |     _
    //
    char theBoard[ BOARD_SIZE][ NUMBER_OF_ELEMENTS];
    // Declare the 4 robots, which will store the board position of where they are found
    // Robots are displayed using values 1-4.  We declare 5 robots, though we will never use
    // the 0th robot in the array, so that we can use the robot digit itself as the index.
    int theRobots[ NUMBER_OF_ROBOTS + 1];
    // Number of destination pieces. Value comes from the data file.
    int numberOfDestinationPieces;
    int goalRobot;                  // the robot that needs to reach the goal
    char destinationLetter;         // the goal letter

};


#endif
